@@ -20,8 +20,8 @@ mod win_x86_64;
 use crate::nasm_builder::NasmBuilder;
 use crate::{
     cargo_env, disable_jitter_entropy, emit_warning, env_var_to_bool, execute_command,
-    get_crate_cflags, is_no_asm, optional_env_optional_crate_target, optional_env_target, out_dir,
-    requested_c_std, set_env_for_target, target, target_arch, target_env, target_os, target_vendor,
+    get_crate_cc, get_crate_cflags, get_crate_cxx, is_no_asm, out_dir, requested_c_std,
+    set_env_for_target, target, target_arch, target_env, target_os, target_vendor,
     test_clang_cl_command, CStdRequested, OutputLibType,
 };
 use std::cell::Cell;
@@ -211,10 +211,10 @@ impl CcBuilder {
             }
         }
 
-        if let Some(cc) = optional_env_optional_crate_target("CC") {
+        if let Some(cc) = get_crate_cc() {
             set_env_for_target("CC", &cc);
         }
-        if let Some(cxx) = optional_env_optional_crate_target("CXX") {
+        if let Some(cxx) = get_crate_cxx() {
             set_env_for_target("CXX", &cxx);
         }
 
@@ -358,8 +358,7 @@ impl CcBuilder {
     }
 
     pub fn prepare_builder(&self) -> cc::Build {
-        let cflags = get_crate_cflags();
-        if !cflags.is_empty() {
+        if let Some(cflags) = get_crate_cflags() {
             set_env_for_target("CFLAGS", cflags);
         }
 
@@ -393,15 +392,30 @@ impl CcBuilder {
 
     #[allow(clippy::unused_self)]
     fn add_defines(&self, build_options: &mut Vec<BuildOption>, is_like_msvc: bool) {
-        if is_like_msvc {
-            build_options.push(BuildOption::define("_HAS_EXCEPTIONS", "0"));
+        // WIN32_LEAN_AND_MEAN and NOMINMAX are needed for all Windows targets to avoid
+        // header type definition errors, no matter the compiler. This matches the behavior
+        // in aws-lc/CMakeLists.txt, which defines these for all WIN32 targets
+        if target_os() == "windows" {
             build_options.push(BuildOption::define("WIN32_LEAN_AND_MEAN", ""));
             build_options.push(BuildOption::define("NOMINMAX", ""));
+        }
+
+        if is_like_msvc {
+            build_options.push(BuildOption::define("_HAS_EXCEPTIONS", "0"));
             build_options.push(BuildOption::define("_CRT_SECURE_NO_WARNINGS", "0"));
             build_options.push(BuildOption::define(
                 "_STL_EXTRA_DISABLED_WARNINGS",
                 "4774 4987",
             ));
+
+            if target().ends_with("-win7-windows-msvc") {
+                // 0x0601 is the value of `_WIN32_WINNT_WIN7`
+                build_options.push(BuildOption::define("_WIN32_WINNT", "0x0601"));
+                emit_warning(format!(
+                    "Setting _WIN32_WINNT to _WIN32_WINNT_WIN7 for {} target",
+                    target()
+                ));
+            }
         }
     }
 
@@ -416,7 +430,7 @@ impl CcBuilder {
             option.apply_cc(&mut je_builder);
         }
 
-        if let Some(original_cflags) = optional_env_target("CFLAGS") {
+        if let Some(original_cflags) = get_crate_cflags() {
             let mut new_cflags = original_cflags.clone();
             if is_like_msvc {
                 new_cflags.push_str(" -Od");
@@ -435,7 +449,10 @@ impl CcBuilder {
             // Certain MacOS system headers are guarded by _POSIX_C_SOURCE and _DARWIN_C_SOURCE
             je_builder.define("_DARWIN_C_SOURCE", "1");
         }
-        je_builder.pic(true);
+        // Only enable PIC on non-Windows targets. Windows doesn't support -fPIC.
+        if target_os() != "windows" {
+            je_builder.pic(true);
+        }
         if is_like_msvc {
             je_builder.flag("-Od").flag("-W4").flag("-DYNAMICBASE");
         } else {
@@ -636,6 +653,14 @@ impl CcBuilder {
     // This should be kept in alignment with the same check performed by the CMake build.
     // See: https://github.com/search?q=repo%3Aaws%2Faws-lc%20check_run&type=code
     fn memcmp_check(&self) {
+        // This check compiles, links, and executes a test program. When cross-compiling
+        // (HOST != TARGET), we cannot execute the resulting binary, so we skip this check.
+        // This also avoids linker configuration issues with cross-compilation toolchains
+        // (e.g., cross-rs Darwin toolchains that set invalid -fuse-ld= flags in CFLAGS).
+        if cargo_env("HOST") != target() {
+            return;
+        }
+
         let basename = "memcmp_invalid_stripped_check";
         let exec_path = out_dir().join(basename);
         let memcmp_build = cc::Build::default();
@@ -690,24 +715,21 @@ impl CcBuilder {
             memcmp_compile_result.stdout
         );
 
-        // We can only execute the binary when the host and target platforms match.
-        if cargo_env("HOST") == target() {
-            let result = execute_command(exec_path.as_os_str(), &[]);
-            assert!(
-                result.status,
-                "### COMPILER BUG DETECTED ###\nYour compiler ({}) is not supported due to a memcmp related bug reported in \
-                https://gcc.gnu.org/bugzilla/show_bug.cgi?id=95189. \
-                We strongly recommend against using this compiler. \n\
-                EXECUTED: {}\n\
-                ERROR: {}\n\
-                OUTPUT: {}\n\
-                ",
-                memcmp_compiler.path().display(),
-                memcmp_compile_result.executed,
-                memcmp_compile_result.stderr,
-                memcmp_compile_result.stdout
-            );
-        }
+        let result = execute_command(exec_path.as_os_str(), &[]);
+        assert!(
+            result.status,
+            "### COMPILER BUG DETECTED ###\nYour compiler ({}) is not supported due to a memcmp related bug reported in \
+            https://gcc.gnu.org/bugzilla/show_bug.cgi?id=95189. \
+            We strongly recommend against using this compiler. \n\
+            EXECUTED: {}\n\
+            ERROR: {}\n\
+            OUTPUT: {}\n\
+            ",
+            memcmp_compiler.path().display(),
+            memcmp_compile_result.executed,
+            memcmp_compile_result.stderr,
+            memcmp_compile_result.stdout
+        );
         let _ = fs::remove_file(exec_path);
     }
     fn run_compiler_checks(&self, cc_build: &mut cc::Build) {
@@ -725,7 +747,7 @@ impl CcBuilder {
             self.compiler_features.set(compiler_features);
             cc_build.define("MY_ASSEMBLER_SUPPORTS_NEON_SHA3_EXTENSION", Some("1"));
         }
-        if target_os() == "linux" {
+        if target_os() == "linux" || target_os() == "android" {
             if self.compiler_check("linux_random_h", Vec::<&'static str>::new()) {
                 cc_build.define("HAVE_LINUX_RANDOM_H", Some("1"));
             } else if self.compiler_check("linux_random_h", vec!["-DDEFINE_U32"]) {
@@ -759,7 +781,7 @@ impl crate::Builder for CcBuilder {
         if target_os() == "windows"
             && target_arch() == "aarch64"
             && target_env() == "msvc"
-            && optional_env_optional_crate_target("CC").is_none()
+            && get_crate_cc().is_none()
             && test_clang_cl_command()
         {
             set_env_for_target("CC", "clang-cl");
