@@ -162,6 +162,20 @@ s! {
         __unused8: Padding<*mut c_void>,
     }
 
+    pub struct tm {
+        pub tm_sec: c_int,
+        pub tm_min: c_int,
+        pub tm_hour: c_int,
+        pub tm_mday: c_int,
+        pub tm_mon: c_int,
+        pub tm_year: c_int,
+        pub tm_wday: c_int,
+        pub tm_yday: c_int,
+        pub tm_isdst: c_int,
+        pub tm_gmtoff: c_long,
+        pub tm_zone: *mut c_char,
+    }
+
     pub struct mq_attr {
         pub mq_flags: c_long,
         pub mq_maxmsg: c_long,
@@ -359,7 +373,7 @@ s! {
     }
 
     pub struct uucred {
-        pub cr_unused: c_ushort,
+        cr_unused: Padding<c_ushort>,
         pub cr_uid: crate::uid_t,
         pub cr_gid: crate::gid_t,
         pub cr_ngroups: c_short,
@@ -2181,12 +2195,6 @@ extern "C" {
         ts: *const crate::timespec,
         sigmask: *const crate::sigset_t,
     ) -> c_int;
-    pub fn ppoll(
-        fds: *mut crate::pollfd,
-        nfds: crate::nfds_t,
-        ts: *const crate::timespec,
-        sigmask: *const crate::sigset_t,
-    ) -> c_int;
     pub fn getrandom(buf: *mut c_void, buflen: size_t, flags: c_uint) -> ssize_t;
 
     pub fn reboot(mode: c_int, bootstr: *mut c_char) -> c_int;
