@@ -19,8 +19,9 @@ use crate::formatting::Formattable;
 use crate::internal_macros::{carry, cascade, const_try, const_try_opt, div_floor, ensure_ranged};
 #[cfg(feature = "parsing")]
 use crate::parsing::Parsable;
+use crate::util::days_in_year;
 use crate::{
-    error, util, Date, Duration, Month, OffsetDateTime, PrimitiveDateTime, Time, UtcOffset, Weekday,
+    Date, Duration, Month, OffsetDateTime, PrimitiveDateTime, Time, UtcOffset, Weekday, error,
 };
 
 /// The Julian day of the Unix epoch.
@@ -327,33 +328,33 @@ impl UtcDateTime {
     #[inline]
     pub(crate) const fn to_offset_raw(self, offset: UtcOffset) -> (i32, u16, Time) {
         let (second, carry) = carry!(@most_once
-            self.second() as i8 + offset.seconds_past_minute(),
+            self.second().cast_signed() + offset.seconds_past_minute(),
             0..Second::per_t(Minute)
         );
         let (minute, carry) = carry!(@most_once
-            self.minute() as i8 + offset.minutes_past_hour() + carry,
+            self.minute().cast_signed() + offset.minutes_past_hour() + carry,
             0..Minute::per_t(Hour)
         );
         let (hour, carry) = carry!(@most_twice
-            self.hour() as i8 + offset.whole_hours() + carry,
+            self.hour().cast_signed() + offset.whole_hours() + carry,
             0..Hour::per_t(Day)
         );
         let (mut year, ordinal) = self.to_ordinal_date();
-        let mut ordinal = ordinal as i16 + carry;
+        let mut ordinal = ordinal.cast_signed() + carry;
         cascade!(ordinal => year);
 
         debug_assert!(ordinal > 0);
-        debug_assert!(ordinal <= util::days_in_year(year) as i16);
+        debug_assert!(ordinal <= days_in_year(year).cast_signed());
 
         (
             year,
-            ordinal as u16,
+            ordinal.cast_unsigned(),
             // Safety: The cascades above ensure the values are in range.
             unsafe {
                 Time::__from_hms_nanos_unchecked(
-                    hour as u8,
-                    minute as u8,
-                    second as u8,
+                    hour.cast_unsigned(),
+                    minute.cast_unsigned(),
+                    second.cast_unsigned(),
                     self.nanosecond(),
                 )
             },
@@ -600,9 +601,6 @@ impl UtcDateTime {
 
     /// Get the Julian day for the date. The time is not taken into account for this calculation.
     ///
-    /// The algorithm to perform this conversion is derived from one provided by Peter Baum; it is
-    /// freely available [here](https://www.researchgate.net/publication/316558298_Date_Algorithms).
-    ///
     /// ```rust
     /// # use time_macros::utc_datetime;
     /// assert_eq!(utc_datetime!(-4713-11-24 0:00).to_julian_day(), 0);
@@ -770,15 +768,15 @@ impl UtcDateTime {
     /// assert_eq!(UtcDateTime::MIN.checked_add((-2).days()), None);
     /// assert_eq!(UtcDateTime::MAX.checked_add(1.days()), None);
     /// assert_eq!(
-    ///     utc_datetime!(2019 - 11 - 25 15:30).checked_add(27.hours()),
-    ///     Some(utc_datetime!(2019 - 11 - 26 18:30))
+    ///     utc_datetime!(2019-11-25 15:30).checked_add(27.hours()),
+    ///     Some(utc_datetime!(2019-11-26 18:30))
     /// );
     /// ```
     #[inline]
     pub const fn checked_add(self, duration: Duration) -> Option<Self> {
-        Some(Self::from_primitive(const_try_opt!(self
-            .inner
-            .checked_add(duration))))
+        Some(Self::from_primitive(const_try_opt!(
+            self.inner.checked_add(duration)
+        )))
     }
 
     /// Computes `self - duration`, returning `None` if an overflow occurred.
@@ -789,15 +787,15 @@ impl UtcDateTime {
     /// assert_eq!(UtcDateTime::MIN.checked_sub(2.days()), None);
     /// assert_eq!(UtcDateTime::MAX.checked_sub((-1).days()), None);
     /// assert_eq!(
-    ///     utc_datetime!(2019 - 11 - 25 15:30).checked_sub(27.hours()),
-    ///     Some(utc_datetime!(2019 - 11 - 24 12:30))
+    ///     utc_datetime!(2019-11-25 15:30).checked_sub(27.hours()),
+    ///     Some(utc_datetime!(2019-11-24 12:30))
     /// );
     /// ```
     #[inline]
     pub const fn checked_sub(self, duration: Duration) -> Option<Self> {
-        Some(Self::from_primitive(const_try_opt!(self
-            .inner
-            .checked_sub(duration))))
+        Some(Self::from_primitive(const_try_opt!(
+            self.inner.checked_sub(duration)
+        )))
     }
 
     /// Computes `self + duration`, saturating value on overflow.
@@ -814,8 +812,8 @@ impl UtcDateTime {
     ///     UtcDateTime::MAX
     /// );
     /// assert_eq!(
-    ///     utc_datetime!(2019 - 11 - 25 15:30).saturating_add(27.hours()),
-    ///     utc_datetime!(2019 - 11 - 26 18:30)
+    ///     utc_datetime!(2019-11-25 15:30).saturating_add(27.hours()),
+    ///     utc_datetime!(2019-11-26 18:30)
     /// );
     /// ```
     #[inline]
@@ -837,8 +835,8 @@ impl UtcDateTime {
     ///     UtcDateTime::MAX
     /// );
     /// assert_eq!(
-    ///     utc_datetime!(2019 - 11 - 25 15:30).saturating_sub(27.hours()),
-    ///     utc_datetime!(2019 - 11 - 24 12:30)
+    ///     utc_datetime!(2019-11-25 15:30).saturating_sub(27.hours()),
+    ///     utc_datetime!(2019-11-24 12:30)
     /// );
     /// ```
     #[inline]
@@ -884,18 +882,18 @@ impl UtcDateTime {
     /// ```rust
     /// # use time_macros::utc_datetime;
     /// assert_eq!(
-    ///     utc_datetime!(2022 - 02 - 18 12:00).replace_year(2019),
-    ///     Ok(utc_datetime!(2019 - 02 - 18 12:00))
+    ///     utc_datetime!(2022-02-18 12:00).replace_year(2019),
+    ///     Ok(utc_datetime!(2019-02-18 12:00))
     /// );
-    /// assert!(utc_datetime!(2022 - 02 - 18 12:00).replace_year(-1_000_000_000).is_err()); // -1_000_000_000 isn't a valid year
-    /// assert!(utc_datetime!(2022 - 02 - 18 12:00).replace_year(1_000_000_000).is_err()); // 1_000_000_000 isn't a valid year
+    /// assert!(utc_datetime!(2022-02-18 12:00).replace_year(-1_000_000_000).is_err()); // -1_000_000_000 isn't a valid year
+    /// assert!(utc_datetime!(2022-02-18 12:00).replace_year(1_000_000_000).is_err()); // 1_000_000_000 isn't a valid year
     /// ```
     #[must_use = "This method does not mutate the original `UtcDateTime`."]
     #[inline]
     pub const fn replace_year(self, year: i32) -> Result<Self, error::ComponentRange> {
-        Ok(Self::from_primitive(const_try!(self
-            .inner
-            .replace_year(year))))
+        Ok(Self::from_primitive(const_try!(
+            self.inner.replace_year(year)
+        )))
     }
 
     /// Replace the month of the year.
@@ -904,17 +902,17 @@ impl UtcDateTime {
     /// # use time_macros::utc_datetime;
     /// # use time::Month;
     /// assert_eq!(
-    ///     utc_datetime!(2022 - 02 - 18 12:00).replace_month(Month::January),
-    ///     Ok(utc_datetime!(2022 - 01 - 18 12:00))
+    ///     utc_datetime!(2022-02-18 12:00).replace_month(Month::January),
+    ///     Ok(utc_datetime!(2022-01-18 12:00))
     /// );
-    /// assert!(utc_datetime!(2022 - 01 - 30 12:00).replace_month(Month::February).is_err()); // 30 isn't a valid day in February
+    /// assert!(utc_datetime!(2022-01-30 12:00).replace_month(Month::February).is_err()); // 30 isn't a valid day in February
     /// ```
     #[must_use = "This method does not mutate the original `UtcDateTime`."]
     #[inline]
     pub const fn replace_month(self, month: Month) -> Result<Self, error::ComponentRange> {
-        Ok(Self::from_primitive(const_try!(self
-            .inner
-            .replace_month(month))))
+        Ok(Self::from_primitive(const_try!(
+            self.inner.replace_month(month)
+        )))
     }
 
     /// Replace the day of the month.
@@ -922,18 +920,18 @@ impl UtcDateTime {
     /// ```rust
     /// # use time_macros::utc_datetime;
     /// assert_eq!(
-    ///     utc_datetime!(2022 - 02 - 18 12:00).replace_day(1),
-    ///     Ok(utc_datetime!(2022 - 02 - 01 12:00))
+    ///     utc_datetime!(2022-02-18 12:00).replace_day(1),
+    ///     Ok(utc_datetime!(2022-02-01 12:00))
     /// );
-    /// assert!(utc_datetime!(2022 - 02 - 18 12:00).replace_day(0).is_err()); // 00 isn't a valid day
-    /// assert!(utc_datetime!(2022 - 02 - 18 12:00).replace_day(30).is_err()); // 30 isn't a valid day in February
+    /// assert!(utc_datetime!(2022-02-18 12:00).replace_day(0).is_err()); // 00 isn't a valid day
+    /// assert!(utc_datetime!(2022-02-18 12:00).replace_day(30).is_err()); // 30 isn't a valid day in February
     /// ```
     #[must_use = "This method does not mutate the original `UtcDateTime`."]
     #[inline]
     pub const fn replace_day(self, day: u8) -> Result<Self, error::ComponentRange> {
-        Ok(Self::from_primitive(const_try!(self
-            .inner
-            .replace_day(day))))
+        Ok(Self::from_primitive(const_try!(
+            self.inner.replace_day(day)
+        )))
     }
 
     /// Replace the day of the year.
@@ -947,9 +945,24 @@ impl UtcDateTime {
     #[must_use = "This method does not mutate the original `UtcDateTime`."]
     #[inline]
     pub const fn replace_ordinal(self, ordinal: u16) -> Result<Self, error::ComponentRange> {
-        Ok(Self::from_primitive(const_try!(self
-            .inner
-            .replace_ordinal(ordinal))))
+        Ok(Self::from_primitive(const_try!(
+            self.inner.replace_ordinal(ordinal)
+        )))
+    }
+
+    /// Truncate to the start of the day, setting the time to midnight.
+    ///
+    /// ```rust
+    /// # use time_macros::utc_datetime;
+    /// assert_eq!(
+    ///     utc_datetime!(2022-02-18 15:30:45.123_456_789).truncate_to_day(),
+    ///     utc_datetime!(2022-02-18 0:00)
+    /// );
+    /// ```
+    #[must_use = "This method does not mutate the original `UtcDateTime`."]
+    #[inline]
+    pub const fn truncate_to_day(self) -> Self {
+        Self::from_primitive(self.inner.truncate_to_day())
     }
 
     /// Replace the clock hour.
@@ -957,17 +970,32 @@ impl UtcDateTime {
     /// ```rust
     /// # use time_macros::utc_datetime;
     /// assert_eq!(
-    ///     utc_datetime!(2022 - 02 - 18 01:02:03.004_005_006).replace_hour(7),
-    ///     Ok(utc_datetime!(2022 - 02 - 18 07:02:03.004_005_006))
+    ///     utc_datetime!(2022-02-18 01:02:03.004_005_006).replace_hour(7),
+    ///     Ok(utc_datetime!(2022-02-18 07:02:03.004_005_006))
     /// );
-    /// assert!(utc_datetime!(2022 - 02 - 18 01:02:03.004_005_006).replace_hour(24).is_err()); // 24 isn't a valid hour
+    /// assert!(utc_datetime!(2022-02-18 01:02:03.004_005_006).replace_hour(24).is_err()); // 24 isn't a valid hour
     /// ```
     #[must_use = "This method does not mutate the original `UtcDateTime`."]
     #[inline]
     pub const fn replace_hour(self, hour: u8) -> Result<Self, error::ComponentRange> {
-        Ok(Self::from_primitive(const_try!(self
-            .inner
-            .replace_hour(hour))))
+        Ok(Self::from_primitive(const_try!(
+            self.inner.replace_hour(hour)
+        )))
+    }
+
+    /// Truncate to the hour, setting the minute, second, and subsecond components to zero.
+    ///
+    /// ```rust
+    /// # use time_macros::utc_datetime;
+    /// assert_eq!(
+    ///     utc_datetime!(2022-02-18 15:30:45.123_456_789).truncate_to_hour(),
+    ///     utc_datetime!(2022-02-18 15:00)
+    /// );
+    /// ```
+    #[must_use = "This method does not mutate the original `UtcDateTime`."]
+    #[inline]
+    pub const fn truncate_to_hour(self) -> Self {
+        Self::from_primitive(self.inner.truncate_to_hour())
     }
 
     /// Replace the minutes within the hour.
@@ -975,10 +1003,10 @@ impl UtcDateTime {
     /// ```rust
     /// # use time_macros::utc_datetime;
     /// assert_eq!(
-    ///     utc_datetime!(2022 - 02 - 18 01:02:03.004_005_006).replace_minute(7),
-    ///     Ok(utc_datetime!(2022 - 02 - 18 01:07:03.004_005_006))
+    ///     utc_datetime!(2022-02-18 01:02:03.004_005_006).replace_minute(7),
+    ///     Ok(utc_datetime!(2022-02-18 01:07:03.004_005_006))
     /// );
-    /// assert!(utc_datetime!(2022 - 02 - 18 01:02:03.004_005_006).replace_minute(60).is_err()); // 60 isn't a valid minute
+    /// assert!(utc_datetime!(2022-02-18 01:02:03.004_005_006).replace_minute(60).is_err()); // 60 isn't a valid minute
     /// ```
     #[must_use = "This method does not mutate the original `UtcDateTime`."]
     #[inline]
@@ -986,9 +1014,24 @@ impl UtcDateTime {
         self,
         sunday_based_week: u8,
     ) -> Result<Self, error::ComponentRange> {
-        Ok(Self::from_primitive(const_try!(self
-            .inner
-            .replace_minute(sunday_based_week))))
+        Ok(Self::from_primitive(const_try!(
+            self.inner.replace_minute(sunday_based_week)
+        )))
+    }
+
+    /// Truncate to the minute, setting the second and subsecond components to zero.
+    ///
+    /// ```rust
+    /// # use time_macros::utc_datetime;
+    /// assert_eq!(
+    ///     utc_datetime!(2022-02-18 15:30:45.123_456_789).truncate_to_minute(),
+    ///     utc_datetime!(2022-02-18 15:30)
+    /// );
+    /// ```
+    #[must_use = "This method does not mutate the original `UtcDateTime`."]
+    #[inline]
+    pub const fn truncate_to_minute(self) -> Self {
+        Self::from_primitive(self.inner.truncate_to_minute())
     }
 
     /// Replace the seconds within the minute.
@@ -996,10 +1039,10 @@ impl UtcDateTime {
     /// ```rust
     /// # use time_macros::utc_datetime;
     /// assert_eq!(
-    ///     utc_datetime!(2022 - 02 - 18 01:02:03.004_005_006).replace_second(7),
-    ///     Ok(utc_datetime!(2022 - 02 - 18 01:02:07.004_005_006))
+    ///     utc_datetime!(2022-02-18 01:02:03.004_005_006).replace_second(7),
+    ///     Ok(utc_datetime!(2022-02-18 01:02:07.004_005_006))
     /// );
-    /// assert!(utc_datetime!(2022 - 02 - 18 01:02:03.004_005_006).replace_second(60).is_err()); // 60 isn't a valid second
+    /// assert!(utc_datetime!(2022-02-18 01:02:03.004_005_006).replace_second(60).is_err()); // 60 isn't a valid second
     /// ```
     #[must_use = "This method does not mutate the original `UtcDateTime`."]
     #[inline]
@@ -1007,9 +1050,24 @@ impl UtcDateTime {
         self,
         monday_based_week: u8,
     ) -> Result<Self, error::ComponentRange> {
-        Ok(Self::from_primitive(const_try!(self
-            .inner
-            .replace_second(monday_based_week))))
+        Ok(Self::from_primitive(const_try!(
+            self.inner.replace_second(monday_based_week)
+        )))
+    }
+
+    /// Truncate to the second, setting the subsecond components to zero.
+    ///
+    /// ```rust
+    /// # use time_macros::utc_datetime;
+    /// assert_eq!(
+    ///     utc_datetime!(2022-02-18 15:30:45.123_456_789).truncate_to_second(),
+    ///     utc_datetime!(2022-02-18 15:30:45)
+    /// );
+    /// ```
+    #[must_use = "This method does not mutate the original `UtcDateTime`."]
+    #[inline]
+    pub const fn truncate_to_second(self) -> Self {
+        Self::from_primitive(self.inner.truncate_to_second())
     }
 
     /// Replace the milliseconds within the second.
@@ -1017,10 +1075,10 @@ impl UtcDateTime {
     /// ```rust
     /// # use time_macros::utc_datetime;
     /// assert_eq!(
-    ///     utc_datetime!(2022 - 02 - 18 01:02:03.004_005_006).replace_millisecond(7),
-    ///     Ok(utc_datetime!(2022 - 02 - 18 01:02:03.007))
+    ///     utc_datetime!(2022-02-18 01:02:03.004_005_006).replace_millisecond(7),
+    ///     Ok(utc_datetime!(2022-02-18 01:02:03.007))
     /// );
-    /// assert!(utc_datetime!(2022 - 02 - 18 01:02:03.004_005_006).replace_millisecond(1_000).is_err()); // 1_000 isn't a valid millisecond
+    /// assert!(utc_datetime!(2022-02-18 01:02:03.004_005_006).replace_millisecond(1_000).is_err()); // 1_000 isn't a valid millisecond
     /// ```
     #[must_use = "This method does not mutate the original `UtcDateTime`."]
     #[inline]
@@ -1028,9 +1086,24 @@ impl UtcDateTime {
         self,
         millisecond: u16,
     ) -> Result<Self, error::ComponentRange> {
-        Ok(Self::from_primitive(const_try!(self
-            .inner
-            .replace_millisecond(millisecond))))
+        Ok(Self::from_primitive(const_try!(
+            self.inner.replace_millisecond(millisecond)
+        )))
+    }
+
+    /// Truncate to the millisecond, setting the microsecond and nanosecond components to zero.
+    ///
+    /// ```rust
+    /// # use time_macros::utc_datetime;
+    /// assert_eq!(
+    ///     utc_datetime!(2022-02-18 15:30:45.123_456_789).truncate_to_millisecond(),
+    ///     utc_datetime!(2022-02-18 15:30:45.123)
+    /// );
+    /// ```
+    #[must_use = "This method does not mutate the original `UtcDateTime`."]
+    #[inline]
+    pub const fn truncate_to_millisecond(self) -> Self {
+        Self::from_primitive(self.inner.truncate_to_millisecond())
     }
 
     /// Replace the microseconds within the second.
@@ -1038,10 +1111,10 @@ impl UtcDateTime {
     /// ```rust
     /// # use time_macros::utc_datetime;
     /// assert_eq!(
-    ///     utc_datetime!(2022 - 02 - 18 01:02:03.004_005_006).replace_microsecond(7_008),
-    ///     Ok(utc_datetime!(2022 - 02 - 18 01:02:03.007_008))
+    ///     utc_datetime!(2022-02-18 01:02:03.004_005_006).replace_microsecond(7_008),
+    ///     Ok(utc_datetime!(2022-02-18 01:02:03.007_008))
     /// );
-    /// assert!(utc_datetime!(2022 - 02 - 18 01:02:03.004_005_006).replace_microsecond(1_000_000).is_err()); // 1_000_000 isn't a valid microsecond
+    /// assert!(utc_datetime!(2022-02-18 01:02:03.004_005_006).replace_microsecond(1_000_000).is_err()); // 1_000_000 isn't a valid microsecond
     /// ```
     #[must_use = "This method does not mutate the original `UtcDateTime`."]
     #[inline]
@@ -1049,9 +1122,24 @@ impl UtcDateTime {
         self,
         microsecond: u32,
     ) -> Result<Self, error::ComponentRange> {
-        Ok(Self::from_primitive(const_try!(self
-            .inner
-            .replace_microsecond(microsecond))))
+        Ok(Self::from_primitive(const_try!(
+            self.inner.replace_microsecond(microsecond)
+        )))
+    }
+
+    /// Truncate to the microsecond, setting the nanosecond component to zero.
+    ///
+    /// ```rust
+    /// # use time_macros::utc_datetime;
+    /// assert_eq!(
+    ///     utc_datetime!(2022-02-18 15:30:45.123_456_789).truncate_to_microsecond(),
+    ///     utc_datetime!(2022-02-18 15:30:45.123_456)
+    /// );
+    /// ```
+    #[must_use = "This method does not mutate the original `UtcDateTime`."]
+    #[inline]
+    pub const fn truncate_to_microsecond(self) -> Self {
+        Self::from_primitive(self.inner.truncate_to_microsecond())
     }
 
     /// Replace the nanoseconds within the second.
@@ -1059,17 +1147,17 @@ impl UtcDateTime {
     /// ```rust
     /// # use time_macros::utc_datetime;
     /// assert_eq!(
-    ///     utc_datetime!(2022 - 02 - 18 01:02:03.004_005_006).replace_nanosecond(7_008_009),
-    ///     Ok(utc_datetime!(2022 - 02 - 18 01:02:03.007_008_009))
+    ///     utc_datetime!(2022-02-18 01:02:03.004_005_006).replace_nanosecond(7_008_009),
+    ///     Ok(utc_datetime!(2022-02-18 01:02:03.007_008_009))
     /// );
-    /// assert!(utc_datetime!(2022 - 02 - 18 01:02:03.004_005_006).replace_nanosecond(1_000_000_000).is_err()); // 1_000_000_000 isn't a valid nanosecond
+    /// assert!(utc_datetime!(2022-02-18 01:02:03.004_005_006).replace_nanosecond(1_000_000_000).is_err()); // 1_000_000_000 isn't a valid nanosecond
     /// ```
     #[must_use = "This method does not mutate the original `UtcDateTime`."]
     #[inline]
     pub const fn replace_nanosecond(self, nanosecond: u32) -> Result<Self, error::ComponentRange> {
-        Ok(Self::from_primitive(const_try!(self
-            .inner
-            .replace_nanosecond(nanosecond))))
+        Ok(Self::from_primitive(const_try!(
+            self.inner.replace_nanosecond(nanosecond)
+        )))
     }
 }
 
@@ -1083,12 +1171,7 @@ impl UtcDateTime {
         output: &mut (impl io::Write + ?Sized),
         format: &(impl Formattable + ?Sized),
     ) -> Result<usize, error::Format> {
-        format.format_into(
-            output,
-            Some(self.date()),
-            Some(self.time()),
-            Some(UtcOffset::UTC),
-        )
+        format.format_into(output, &self, &mut Default::default())
     }
 
     /// Format the `UtcDateTime` using the provided [format
@@ -1109,7 +1192,7 @@ impl UtcDateTime {
     /// ```
     #[inline]
     pub fn format(self, format: &(impl Formattable + ?Sized)) -> Result<String, error::Format> {
-        format.format(Some(self.date()), Some(self.time()), Some(UtcOffset::UTC))
+        format.format(&self, &mut Default::default())
     }
 }
 
