@@ -2,7 +2,7 @@ use std::io;
 use std::num::NonZero;
 
 use time::format_description::well_known::iso8601::{DateKind, OffsetPrecision, TimePrecision};
-use time::format_description::well_known::{iso8601, Iso8601, Rfc2822, Rfc3339};
+use time::format_description::well_known::{Iso8601, Rfc2822, Rfc3339, iso8601};
 use time::format_description::{self, BorrowedFormatItem, OwnedFormatItem};
 use time::macros::{date, datetime, format_description as fd, offset, time, utc_datetime};
 use time::{OffsetDateTime, Time};
@@ -118,10 +118,6 @@ fn iso_8601() -> time::Result<()> {
         };
     }
 
-    assert!(std::panic::catch_unwind(|| {
-        let _unused = datetime!(2021-01-02 03:04:05 UTC).format(&Iso8601::PARSING);
-    })
-    .is_err());
     assert_eq!(
         datetime!(-123_456-01-02 03:04:05 UTC).format(
             &Iso8601::<
@@ -190,7 +186,7 @@ fn iso_8601() -> time::Result<()> {
         Err(time::error::Format::InvalidComponent("year"))
     ));
     assert!(matches!(
-        datetime!(+10_000-W 01-1 0:00 UTC).format(
+        datetime!(+10_000-W01-1 0:00 UTC).format(
             &Iso8601::<
                 {
                     iso8601::Config::DEFAULT
@@ -293,16 +289,20 @@ fn format_time() -> time::Result<()> {
             time!(13:02:03.456_789_012).format(format_description)?,
             output
         );
-        assert!(time!(13:02:03.456_789_012)
-            .format_into(&mut io::sink(), format_description)
-            .is_ok());
+        assert!(
+            time!(13:02:03.456_789_012)
+                .format_into(&mut io::sink(), format_description)
+                .is_ok()
+        );
         assert_eq!(
             time!(13:02:03.456_789_012).format(&OwnedFormatItem::from(format_description))?,
             output
         );
-        assert!(time!(13:02:03.456_789_012)
-            .format_into(&mut io::sink(), &OwnedFormatItem::from(format_description))
-            .is_ok());
+        assert!(
+            time!(13:02:03.456_789_012)
+                .format_into(&mut io::sink(), &OwnedFormatItem::from(format_description))
+                .is_ok()
+        );
     }
 
     assert_eq!(
@@ -403,16 +403,20 @@ fn format_date() -> time::Result<()> {
 
     for &(format_description, output) in &format_output {
         assert_eq!(date!(2019-12-31).format(format_description)?, output);
-        assert!(date!(2019-12-31)
-            .format_into(&mut io::sink(), format_description)
-            .is_ok());
+        assert!(
+            date!(2019-12-31)
+                .format_into(&mut io::sink(), format_description)
+                .is_ok()
+        );
         assert_eq!(
             date!(2019-12-31).format(&OwnedFormatItem::from(format_description))?,
             output
         );
-        assert!(date!(2019-12-31)
-            .format_into(&mut io::sink(), &OwnedFormatItem::from(format_description))
-            .is_ok());
+        assert!(
+            date!(2019-12-31)
+                .format_into(&mut io::sink(), &OwnedFormatItem::from(format_description))
+                .is_ok()
+        );
     }
 
     Ok(())
@@ -434,13 +438,13 @@ fn format_date_err() {
 fn display_date() {
     assert_eq!(date!(2019-01-01).to_string(), "2019-01-01");
     assert_eq!(date!(2019-12-31).to_string(), "2019-12-31");
-    assert_eq!(date!(-4713 - 11 - 24).to_string(), "-4713-11-24");
-    assert_eq!(date!(-0001 - 01 - 01).to_string(), "-0001-01-01");
+    assert_eq!(date!(-4713-11-24).to_string(), "-4713-11-24");
+    assert_eq!(date!(-0001-01-01).to_string(), "-0001-01-01");
 
     assert_eq!(date!(+10_000-01-01).to_string(), "+10000-01-01");
     assert_eq!(date!(+100_000-01-01).to_string(), "+100000-01-01");
-    assert_eq!(date!(-10_000 - 01 - 01).to_string(), "-10000-01-01");
-    assert_eq!(date!(-100_000 - 01 - 01).to_string(), "-100000-01-01");
+    assert_eq!(date!(-10_000-01-01).to_string(), "-10000-01-01");
+    assert_eq!(date!(-100_000-01-01).to_string(), "-100000-01-01");
 }
 
 #[test]
@@ -472,16 +476,20 @@ fn format_offset() -> time::Result<()> {
 
     for &(value, format_description, output) in &value_format_output {
         assert_eq!(value.format(format_description)?, output);
-        assert!(value
-            .format_into(&mut io::sink(), format_description)
-            .is_ok());
+        assert!(
+            value
+                .format_into(&mut io::sink(), format_description)
+                .is_ok()
+        );
         assert_eq!(
             value.format(&OwnedFormatItem::from(format_description))?,
             output
         );
-        assert!(value
-            .format_into(&mut io::sink(), &OwnedFormatItem::from(format_description))
-            .is_ok());
+        assert!(
+            value
+                .format_into(&mut io::sink(), &OwnedFormatItem::from(format_description))
+                .is_ok()
+        );
     }
 
     Ok(())
@@ -511,16 +519,20 @@ fn format_pdt() -> time::Result<()> {
         datetime!(1970-01-01 0:00).format(format_description)?,
         "1970-01-01 00:00:00.0"
     );
-    assert!(datetime!(1970-01-01 0:00)
-        .format_into(&mut io::sink(), format_description)
-        .is_ok());
+    assert!(
+        datetime!(1970-01-01 0:00)
+            .format_into(&mut io::sink(), format_description)
+            .is_ok()
+    );
     assert_eq!(
         datetime!(1970-01-01 0:00).format(&OwnedFormatItem::from(format_description))?,
         "1970-01-01 00:00:00.0"
     );
-    assert!(datetime!(1970-01-01 0:00)
-        .format_into(&mut io::sink(), &OwnedFormatItem::from(format_description))
-        .is_ok());
+    assert!(
+        datetime!(1970-01-01 0:00)
+            .format_into(&mut io::sink(), &OwnedFormatItem::from(format_description))
+            .is_ok()
+    );
 
     Ok(())
 }
@@ -548,16 +560,20 @@ fn format_odt() -> time::Result<()> {
         datetime!(1970-01-01 0:00 UTC).format(&format_description)?,
         "1970-01-01 00:00:00.0 +00:00:00"
     );
-    assert!(datetime!(1970-01-01 0:00 UTC)
-        .format_into(&mut io::sink(), &format_description)
-        .is_ok());
+    assert!(
+        datetime!(1970-01-01 0:00 UTC)
+            .format_into(&mut io::sink(), &format_description)
+            .is_ok()
+    );
     assert_eq!(
         datetime!(1970-01-01 0:00 UTC).format(&OwnedFormatItem::from(&format_description))?,
         "1970-01-01 00:00:00.0 +00:00:00"
     );
-    assert!(datetime!(1970-01-01 0:00 UTC)
-        .format_into(&mut io::sink(), &OwnedFormatItem::from(format_description))
-        .is_ok());
+    assert!(
+        datetime!(1970-01-01 0:00 UTC)
+            .format_into(&mut io::sink(), &OwnedFormatItem::from(format_description))
+            .is_ok()
+    );
 
     Ok(())
 }
@@ -578,16 +594,20 @@ fn format_udt() -> time::Result<()> {
         utc_datetime!(1970-01-01 0:00).format(format_description)?,
         "1970-01-01 00:00:00.0"
     );
-    assert!(utc_datetime!(1970-01-01 0:00)
-        .format_into(&mut io::sink(), format_description)
-        .is_ok());
+    assert!(
+        utc_datetime!(1970-01-01 0:00)
+            .format_into(&mut io::sink(), format_description)
+            .is_ok()
+    );
     assert_eq!(
         utc_datetime!(1970-01-01 0:00).format(&OwnedFormatItem::from(format_description))?,
         "1970-01-01 00:00:00.0"
     );
-    assert!(utc_datetime!(1970-01-01 0:00)
-        .format_into(&mut io::sink(), &OwnedFormatItem::from(format_description))
-        .is_ok());
+    assert!(
+        utc_datetime!(1970-01-01 0:00)
+            .format_into(&mut io::sink(), &OwnedFormatItem::from(format_description))
+            .is_ok()
+    );
 
     Ok(())
 }
@@ -609,18 +629,9 @@ fn insufficient_type_information() {
         ));
     };
     assert_insufficient_type_information(Time::MIDNIGHT.format(fd!("[year]")));
-    assert_insufficient_type_information(Time::MIDNIGHT.format(&Rfc3339));
-    assert_insufficient_type_information(date!(2021-001).format(&Rfc3339));
-    assert_insufficient_type_information(datetime!(2021-001 0:00).format(&Rfc3339));
-    assert_insufficient_type_information(Time::MIDNIGHT.format(&Rfc2822));
-    assert_insufficient_type_information(date!(2021-001).format(&Rfc2822));
-    assert_insufficient_type_information(datetime!(2021-001 0:00).format(&Rfc2822));
     assert_insufficient_type_information(Time::MIDNIGHT.format(&BorrowedFormatItem::First(&[
         BorrowedFormatItem::Compound(fd!("[year]")),
     ])));
-    assert_insufficient_type_information(Time::MIDNIGHT.format(&Iso8601::DEFAULT));
-    assert_insufficient_type_information(date!(2021-001).format(&Iso8601::DEFAULT));
-    assert_insufficient_type_information(datetime!(2021-001 0:00).format(&Iso8601::DEFAULT));
 }
 
 #[expect(clippy::cognitive_complexity, reason = "all test the same thing")]
@@ -741,10 +752,10 @@ fn failed_write() -> time::Result<()> {
     assert_err!(Time::MIDNIGHT, fd!("[hour padding:space]"));
     assert_err!(offset!(+1), fd!("[offset_hour sign:mandatory]"));
     assert_err!(offset!(-1), fd!("[offset_hour]"));
-    assert_err!(date!(-1 - 001), fd!("[year]"));
+    assert_err!(date!(-1-001), fd!("[year]"));
     assert_err!(date!(2021-001), fd!("[year sign:mandatory]"));
-    assert_err!(date!(+999_999 - 001), fd!("[year]"));
-    assert_err!(date!(+99_999 - 001), fd!("[year]"));
+    assert_err!(date!(+999_999-001), fd!("[year]"));
+    assert_err!(date!(+99_999-001), fd!("[year]"));
 
     let component_names = [
         "day",
